# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


@register_command_group(
    "network asg",
)
class __CMDGroup(AAZCommandGroup):
    """Manage application security groups (ASGs).

    You can configure network security as a natural extension of an application's structure, ASG allows you to group virtual machines and define network security policies based on those groups. You can specify an application security group as the source and destination in a NSG security rule. For more information, visit https://docs.microsoft.com/azure/virtual-network/create-network-security-group-preview.
    """
    pass


__all__ = ["__CMDGroup"]
